/* RCS: $Id: inputHelper.c,v 1.2 2006/03/17 04:55:57 kanmy Exp kanmy $ 
 * 
 * Sample C code framework for an input helper for sudoku
 * See ATTENTION STUDENT for places that you will have to replace
 *
 * RCS: $Log: inputHelper.c,v $
 * RCS: Revision 1.2  2006/03/17 04:55:57  kanmy
 * RCS: Fixed bug with usage.
 * RCS:
 * RCS: Revision 1.1  2006/03/17 04:46:57  kanmy
 * RCS: Initial revision
 * RCS:
 */
#include <stdio.h>

#define MAXSIZE 4
#define ERR_NO_MEM -1
#define KBFILE sudoku.pl

char **matrix; 

/* Allocates 2 d array for sudoku matrix
 */
int init () {
  int i; 
  matrix = (char **) malloc (sizeof(char *) * MAXSIZE); 
  if (matrix == NULL) { return ERR_NO_MEM; } 
  for (i = 0; i < MAXSIZE; i++) { 
    matrix[i] = (char *) malloc (sizeof(char) * MAXSIZE); 
    if (matrix[i] == NULL) { return ERR_NO_MEM; } 
  } 
  return 0; 
} 

/* Reads input file subject to specifications on: 
 * http://www.comp.nus.edu.sg/~kanmy/courses/3243_2006/hw-hexSudoku.html
 */
int readInput(FILE *fp) {
  char c;
  int row = 0;
  int col = 0;

  do {
    c = fgetc (fp);
    if (c == '\n') { col = 0; row++; }
    else if (c == EOF) { ; }
    else { matrix[row][col++] = c; }
  } while (c != EOF);
  fclose (fp);
  return 0;
}

/* ATTENTION STUDENT: Here's the code that you will want to replace
 * with your own version.  Be smart about the facts that you define
 * and their order.  The kb file holds the generic rules that you've
 * defined for solving the su doku problem.  The output of this
 * program is a script that you can run in prolog.  It should output
 * 1) the kb specific to your problem, 2) a query that will ask prolog
 * to solve the problem, 3) output statements (connected to the query
 * by the "and" connective ',') to create the output for the
 * outputHelper program.
 */
int writeOutput(FILE *op) {
  int i,j;

  fprintf (op, "consult('KBFILE').\n"); /* load the generic rules / strategied for solving hex sudoku */

  /* part 1 - define facts to assert() into the prolog kb that are specific to this su doku instance */
  /* REPLACE ME WITH SOMETHING REASONABLE - this code here is not (necessarily) optimized */
  fprintf (op, "solve(\n");
  for (i = 0; i < MAXSIZE; i++) {
    for (j = 0; j < MAXSIZE; j++) {
      if (matrix[i][j] == '-') 
	fprintf (op,"%c%c", 'A'+j, 'A'+i);	
      else
	fprintf (op,"%c", matrix[i][j]);
      if ((i == MAXSIZE-1) && (j == MAXSIZE-1))
	fprintf (op,")");
      fprintf (op,",");
    }
   fprintf(op,"\n");
  }
  fprintf(op,"\n");

  /* part 2 - ask a query to solve the su doku problem */
  /* REPLACE ME WITH SOMETHING REASONABLE */
  fprintf (op,"tell('prologOutput.txt'),\n");
  
  /* part 3 - continue query with output statements suitable for loading in the outputHelper program */
  /* REPLACE ME WITH SOMETHING REASONABLE */
  
  for (i = 0; i < MAXSIZE; i++) {
    for (j = 0; j < MAXSIZE; j++) {
      if (matrix[i][j] == '-') 
	fprintf (op,"write('%c%c = '), write(%c%c),nl,\n", 'A'+j, 'A'+i, 'A'+j, 'A'+i);
    }
  }  

  fprintf (op,"told.\n");
  fprintf (op,"halt.\n");		/* we're done now */

  fclose(op);
}

/* Usage message, run if invoked incorrectly
 */
void usage(char *progName) {
  fprintf (stderr, "%s usage error:\n", progName);
  fprintf (stderr, "\tUse \"%s <input_file>\" or \"%s < <input_file>\"\n\n", progName, progName);
}

/* Main procedure :
 *  - canonicalizes input from cmd line or stdin
 *  - calls init to allocate memory
 *  - reads input
 *  - processes output 
 */
int main (int argc, char **argv) {
  FILE *fp;			/* file pointer to input file */
  FILE *op;
  int flag = 0;			/* stores status: 0 ok, !0 problem */

  flag = init();		/* call init for dynamic memory allocation */

  if (argc == 2) {		/* canoncailize input */
    fp = fopen (argv[1],"r");
    op = fopen ("sudoku.pl", "w");
    if (fp != NULL) {
    } else {
      fprintf (stderr, "%s fatal: can't open input file \"%s\"!\n", argv[0], argv[1]);
      flag = 1;
    }
    
  } else if (argc == 1) {
    fp = stdin;
  } else {
    flag = 1;
    usage(argv[0]);
  }

  if (flag == 0) {		/* read input */
    flag = readInput(fp);
  }

  if (flag == 0) {		/* write output as prolog KB */
    flag = writeOutput(op);
  }
}
